#ifndef __UTILS_H__
#define __UTILS_H__

#include <stdio.h>
#include <stdint.h>

/** \brief Convert string to IPv4 address.
 *
 *  \param ip Pointer to destination uint32_t value that will store
 *         ip. It must be not NULL.
 *  \param str String to convert.
 *
 *  \return 0 in case of success, error code in other cases.
 */
int str2ip(uint32_t *ip, const char *str);

/** \brief Convert string to MAC address.
 *
 *  \param mac Pointer to destination array that will store
 *         ip. It must be not NULL and must have size to store at least 6 bytes.
 *  \param str String to convert.
 *
 *  \return 0 in case of success, error code in other cases.
 */
int str2mac(unsigned char *mac, const char *str);

/** \brief Convert hexadecimal string to unsigned long value.
 *
 *  \param val Pointer to destination unsigned long value. It must be not NULL.
 *  \param str Hexadeximal string to convert. It can contains "0x" in the
 *             beginning.
 *
 *  \return 0 in case of success, error code in other cases.
 */
int strhex2ulong(unsigned long *val, const char *str);

/** \brief Convert string to unsigned long value.
 *
 *  \param val Pointer to destination unsigned long value. It must be not NULL.
 *  \param str String to convert.
 *
 *  \return 0 in case of success, error code in other cases.
 */
int str2ulong(unsigned long *val, const char *str);

/** \brief Convert string to unsigned short value.
 *
 *  \param val Pointer to destination unsigned short value. It must be not NULL.
 *  \param str String to convert.
 *
 *  \return 0 in case of success, error code in other cases.
 */
int str2ushort(unsigned short *val, const char *str);

/** \brief Convert string to float value.
 *
 *  \param val Pointer to destination float value. It must be not NULL.
 *  \param str String to convert.
 *
 *  \return 0 in case of success, error code in other cases.
 */
int str2float(float *val, const char *str);

/** \brief Convert string to 4-byte user pattern.
 *
 *  \param user_pattern Pointer to destination uint32_t value that will store
 *         user_pattern. It must be not NULL.
 *  \param str String to convert.
 *
 *  \return 0 in case of success, error code in other cases.
 */
int str2up(uint32_t *user_pattern, const char *str);

/** \brief Print MAC adress
 *
 *  \param output Output stream/file
 *  \param addr Pointer to MAC address
 */
void print_mac( FILE *output, const unsigned char *mac_ptr );

/** \brief Print IPv4 address
 *
 *  \param output Output stream/file
 *  \param ip pointer to ip-address
 */
void print_ip( FILE *output, const uint32_t *ip );

/*! \brief Look up for the _exact_ matching of one string to the any from list. 
 *
 *  \param what -- string to find in 
 *  \param where -- list of variants  
 *
 *  \return Index of the string that matching to where or -1 otherwise. 
 **/ 
signed int opt_lookup_case( const char *what, const char * const *where );

/*! \brief Look up for the _exact_ matching of one string to the any from list. 
 *
 *  \param what -- string to find in 
 *  \param where -- list of variants  
 *
 *  \return Index of the string that matching to where or -1 otherwise. 
 **/ 
signed int opt_lookup( const char *what, const char * const *where );

#endif
