-- Declarations
thr_results = {}
thr_trials = {}
lat_results = {}
frl_results = {}
btb_results = {}

local in_rfc2544 = false
local in_results = false
local in_config = false
local in_throughput_results = false
local in_throughput_trials = false
local in_latency_results = false
local in_frameloss_results = false
local in_backtoback_results = false


function parse_rfc2544_report_line(s, parsed)
   if s == nil then
      s = ""
   end
   
   local s_striped = remove_lead_trail_whitespace( s )

   if startswith( s, "RFC2544 test report" ) then
     in_rfc2544 = true 
   elseif in_rfc2544 then
      if startswith( s, "Started" ) then
	local _s = split_space( s ) 
	parsed["rfc_started"] = _s[2] .. " " .. _s[3]
     elseif startswith( s, "Stopped" ) then
	local _s = split_space( s ) 
	parsed["rfc_stopped"] = _s[2] .. " " .. _s[3]
     elseif startswith( s, "Configuration" ) then
        in_config = true
     elseif startswith( s, "Frames" ) and in_config then
       local f = split(s)
       for i = 1, 10 do
         if f[i+1] ~= nil then
           parsed["frames " .. i .. " enable"] = "on" --FIXME
           parsed["frames " .. i .. " size"] = f[i+1]
         else  
	   break
         end
       end  
     elseif startswith( s, "Throughput" ) and in_config then
--Throughput	1s; rate 100 %; resolution  0.01 %; threshold 0 %
       local _s = s:gsub(";", " ")
       local _s = _s:gsub("\t", " ")
       params = split( _s )
       parsed["throughput duration"] = string.sub(params[2],1,-2)
       parsed["throughput maxrate"] = params[4]..params[5]
       parsed["throughput resolution"] = params[7]
       parsed["throughput threshold"] = params[10]
     elseif startswith( s, "Latency" ) and in_config then
--Latency   	1 trials; 1s each; rate_source: Throughput
       local _s = s:gsub(";", " ")
       local _s = _s:gsub("\t", " ")
       params = split( _s )
       parsed["latency count"] = params[2]
       parsed["latency duration"] = string.sub( params[4],1,-2)
       parsed["latency rates src"] = params[7]
     elseif startswith( s, "Frame loss" ) and in_config then
--Frame loss	1s; 9 steps; start rate 100 %; stop rate 10 %
       local _s = s:gsub(";", " ")
       local _s = _s:gsub("\t", " ")
       params = split( _s )
       parsed["frameloss duration"] = string.sub( params[3],1,-2)
       parsed["frameloss steps"] = params[4]
       parsed["frameloss rates start"] = params[8].." "..params[9]
       parsed["frameloss rates stop"] = params[12].." "..params[13]
     elseif startswith( s, "Back-to-back" ) and in_config then
--Back-to-back	1 trials; 2s each; rate_source: Throughput
       local _s = s:gsub(";", " ")
       local _s = _s:gsub("\t", " ")
       params = split( _s )
       parsed["back2back count"] = params[2]
       parsed["back2back duration"] = string.sub(params[4],1,-2)
       parsed["back2back rates src"] = params[7]
     elseif startswith( s,"MAC") and in_config then
       local params = split(s)
       parsed["header src mac"] = params[2]
       parsed["header dst mac"] = params[4]
     elseif startswith( s, "IP address") and in_config then
       local params = split(s)
       parsed["header src ip"] = params[3]
       parsed["header dst ip"] = params[5]
     elseif startswith(s,"UDP") and in_config then
       local params = split(s)
       local udp = split(params[2], "/")
       parsed["header src udp"] = tonumber(udp[1])
       parsed["header dst udp"] = tonumber(udp[2])
     elseif startswith( s, "Results" ) then
       in_config = false
       in_results = true
     end

     if in_results then
        if startswith( remove_lead_trail_whitespace( s ), "Throughput" ) then
           in_throughput_results = true
        elseif startswith( remove_lead_trail_whitespace( s ), "Latency" ) then
           in_latency_results = true
        elseif startswith( remove_lead_trail_whitespace( s ), "Frame loss" ) then
           in_frameloss_results = true
        elseif startswith( remove_lead_trail_whitespace( s ), "Back-to-back" ) then
           in_backtoback_results = true
        end
     end
     
     if in_results then
        local s = remove_lead_trail_whitespace( s )
        if in_throughput_results then
           -- print( s )
           if s == nil or s == "" then
              in_throughput_results = false
           else
              local __s = split_space( s )
              if #__s == 8 and __s[1] ~= "Frame" then
                 -- print( s )
                 table.insert( thr_results, s )
              end
           end
        elseif in_latency_results then
           -- print( s )
           if s == nil or s == "" then
              in_latency_results = false
           else
              local __s = split_space( s )
              if #__s == 4 and __s[1] ~= "Frame" then
                 -- print( s )
                 table.insert( lat_results, s )
              end
           end
        elseif in_frameloss_results then
           -- print( s )
           if s == nil or s == "" then
              in_frameloss_results = false
           else
              local __s = split_space( s )
              if #__s == 7 and __s[1] ~= "Frame" then
                 -- print( s )
                 table.insert( frl_results, s )
              end
           end
        elseif in_backtoback_results then
           if s == nil or s == "" then
              in_backtoback_results = false
              in_rfc2544 = false
           else
              local __s = split_space( s )
              if #__s == 5 and __s[1] ~= "Frame" then
                 -- print( s )
                 table.insert( btb_results, s )
              end
           end
        end
     end
   end

   return parsed
end

function get_rfc2544_report(parsed)
   -- RFC-2544
   return {
      -- RFC-2544 Settings
      settings = {
         
         -- Header
         header = {
            src = {
               mac = parsed["header src mac"],
               ip = parsed["header src ip" ],
               udp = tonumber(parsed["header src udp"]),
            },
            dst = {
               mac = parsed["header dst mac"],
               ip = parsed["header dst ip" ],
               udp = tonumber(parsed["header dst udp"]),
            },
            vlan = {},
            mpls = {},
            tos = "0000", --FIXME
            precedence = 0 --FIXME
         },

         -- Topology
         topology = 0/0, --FIXME

         -- Frames
         frames = (
            function()
               local frames = {}
               for i = 1, 100 do
                  if parsed["frames " .. i .. " enable"] ~= nil then
                     table.insert( frames,
                                   {
                                      enabled = (parsed["frames " .. i .. " enable"] == "on"),
                                      size = tonumber(parsed["frames " .. i .. " size"] ),
                                   } )
                  else
                     break
                  end
               end
               return frames
            end
                  )(),

         -- Config
         config = {
            wait_ms = 1000, --FIXME
            learn_ms = 1000, --FIXME
            check_link = true, --FIXME

            --  Throughput
            throughput = {
               enabled = true, --FIXME
               trial_duration = tonumber(parsed["throughput duration"]),
               maxrate = {
                  value = tonumber(parsed["throughput maxrate"]:match( "^([0-9.,]+)" )) or 100.0,
                  units = parsed["throughput maxrate"]:match( "%s+(.+)$" ) or '%',
               },
               resolution = tonumber(parsed["throughput resolution"]),
               threshold = tonumber(parsed["throughput threshold"]),
            },
--            
            -- Latency
            latency = {
               enabled = true,  --FIXME (parsed["latency enabled"] == "yes"),
               trial_count = tonumber(parsed["latency count"]),
               trial_duration = tonumber(parsed["latency duration"]),
               rates_src = parsed["latency rates src"],
               rates = {} --FIXME
--               rates = (
--                  function()
--                     local rates = {}
--                     for i = 1, 100 do
--                        if parsed["latency rates " .. i] ~= nil then
--                           local value = parsed["latency rates " .. i]
--                           table.insert( rates,
--                                         {
                                            --index = i,
--                                            value = tonumber(value:match( "^([0-9.,]+)" )),
--                                            units = value:match( "%s+(.+)$" )
--                                         } )
--                        else
--                           break
--                        end
--                     end
--                     return rates
--                  end
--                       )(),
            },

            -- Frameloss
            frameloss = {
               enabled = true, --FIXME (parsed["frameloss enabled"] == "yes"),
               trial_duration = tonumber(parsed["frameloss duration"]),
               steps = tonumber(parsed["frameloss steps"]),
               rates_start = {
                  value = tonumber(parsed["frameloss rates start"]:match( "^([0-9.,]+)" )),
                  units = parsed["frameloss rates start"]:match( "%s+(.+)$" ),
               },
               rates_end = {
                  value = tonumber(parsed["frameloss rates stop"]:match( "^([0-9.,]+)" )),
                  units = parsed["frameloss rates stop"]:match( "%s+(.+)$" ),
               },
            },
            
            -- Back-to-back
            backtoback = {
             enabled = true,  --FIXME (parsed["back2back enabled"] == "yes"),
             trial_count = tonumber(parsed["back2back count"]),
               trial_duration = tonumber(parsed["back2back duration"]),
               rates_src = parsed["back2back rates src"],
               rates = {}, --FIXME
--               rates = (
--                  function()
--                     local rates = {}
--                     for i = 1, 100 do
--                        if parsed["back2back rates " .. i] ~= nil then
--                           local value = parsed["back2back rates " .. i]
--                           table.insert( rates,
--                                         {
                                            --index = i,
--                                            value = tonumber(value:match( "^([0-9.,]+)" )),
--                                            units = value:match( "%s+(.+)$" )
--                                         } )
--                        else
--                           break
--                        end
--                     end
--                     return rates
--                  end
--                       )(),
            },
         },
      },

      -- RFC-2544 Info
      info = {
         started = convert_etu_time_to_iso8601( parsed["rfc_started"] ) or null,
         finished = convert_etu_time_to_iso8601( parsed["rfc_stopped"] ) or null,
      },

      -- RFC-2544 Results
      results = {
         -- Throughput results
         throughput = (
            function()
               local frames = {}
               for i,val in ipairs( thr_results ) do
                  local __s = split_space( val )
                  table.insert( frames,
                                {
                                   frame = tonumber( __s[1] ),
                                   ["rate_%"] = tonumber( __s[2] ) or 0,
                                   rate_mbps_l1 = tonumber( __s[3] ) or 0,
                                   rate_mbps_l2 = tonumber( __s[4] ) or 0,
                                   rate_mbps_l3 = tonumber( __s[5] ) or 0,
                                   rate_mbps_l4 = tonumber( __s[6] ) or 0,
                                   frm_per_second = tonumber( __s[7] ) or 0,
                                   status = __s[8] or "Unknown",
                                   trials = {}
                                } );
               end
               return frames
            end
                      )(),

         -- Latency results
         latency = (
            function()
               local frames = {}
               for i,val in ipairs( lat_results ) do
                  local __s = split_space( val )
                  table.insert( frames,
                                {
                                   frame = tonumber( __s[1] ),
                                   ["rate_%"] = tonumber( __s[2] ) or 0,
                                   delay_ms = tonumber( __s[3] ) or 0,
                                   status = __s[4] or "Unknown",
                                   trials = {},
                                } );
               end
               return frames
            end
                   )(),

         -- Frameloss results
         frameloss = (
            function()
               local frames = {}
               for i,val in ipairs( frl_results ) do
                  local __s = split_space( val )
                  table.insert( frames,
                                {
                                   frame = tonumber( __s[1] ),
                                   ["rate_%"] = tonumber( __s[2] ) or 0,
                                   ["loss_%"] = tonumber( __s[3] ) or 0,
                                   loss_mbps_l1 = tonumber( __s[4] ) or 0,
                                   loss_mbps_l2 = tonumber( __s[5] ) or 0,
                                   loss_mbps_l3 = tonumber( __s[6] ) or 0,
                                   loss_mbps_l4 = tonumber( __s[7] ) or 0,
                                   trials = {},
                                } );
               end
               return frames
            end
                   )(),

         -- Back-to-back results
         backtoback = (
            function()
               local frames = {}
               for i,val in ipairs( btb_results ) do
                  local __s = split_space( val )
                  table.insert( frames,
                                {
                                   frame = tonumber( __s[1] ),
                                   ["rate_%"] = tonumber( __s[2] ) or 0,
                                   trial_duration = tonumber( __s[3] ) or 0,
                                   trial_count = tonumber( __s[4] ) or 0,
                                   status = __s[5] or "Unknown",
                                   trials = {},
                                } );
               end
               return frames
            end
                   )(),
      },
   }
end
