import os
import struct
import socket
import sys
import time
  
class DeviceAccess( ):

  def open_dev_file( self ):
    self.fd = os.open( "/dev/etn", os.O_RDWR )

  def close_dev_file( self ):
    os.close( self.fd )

  def read_dev_file( self, addr ):
    os.lseek( self.fd, addr * 2, os.SEEK_SET )
    data_bin = os.read( self.fd, 2 )
    
    unpacker = struct.Struct('H')
    data = unpacker.unpack( data_bin )[0]

    return data
  
  def write_dev_file( self, addr, data ):
    
    packer = struct.Struct( 'H' )
    data_bin = packer.pack( data )
    
    os.lseek( self.fd, addr * 2, os.SEEK_SET )
    os.write( self.fd, data_bin )

#############################################
# Interface select
#############################################

  hold_device    = open_dev_file
  release_device = close_dev_file
  read           = read_dev_file
  write          = write_dev_file


BEEPER_OFFSET          = 207
BEEPER_MAIN_CR         = 0
BEEPER_OSC_PHASE_W0_CR = 0
BEEPER_OSC_PHASE_W1_CR = 0

class Beeper:
  def CR( self, x ):
    return ( BEEPER_OFFSET + x )
  
  def set_freq( self, hz ):
    global BEEPER_MAIN_CR, BEEPER_OSC_PHASE_W0_CR, BEEPER_OSC_PHASE_W1_CR
    phase = pow( 2, 32 ) / 62500000.0 * hz
    phase = int( phase )
  
    low_16 = phase & 0xffff
    hi_16  = ( phase >> 16 ) & 0xffff
  
    self.dev.write( BEEPER_OSC_PHASE_W0_CR, low_16 )
    self.dev.write( BEEPER_OSC_PHASE_W1_CR, hi_16  )
    self.dev.write( BEEPER_MAIN_CR,         0x0000 ) 
    self.dev.write( BEEPER_MAIN_CR,         0x8000 ) 
  
  def set_time( self, ms ):
    global BEEPER_MAIN_CR
    self.dev.write( BEEPER_MAIN_CR, 0x0001 ) 
    time.sleep( ms/1000.0 )
    self.dev.write( BEEPER_MAIN_CR, 0x0000 ) 
  
  def write_32b( self, w0, data_32 ):
    low_16 = data_32 & 0xffff
    hi_16  = ( data_32 >> 16 ) & 0xffff
  
    self.dev.write( w0,     low_16 )
    self.dev.write( w0 + 1, hi_16  )
   
  def set_fpga_time( self, us ):
    ns = us * 1000
    hw_ticks = int( ns / 16 )
    
    # set generation time
    self.write_32b( BEEPER_GEN_TIME_WO_CR, hw_ticks )

    # set run strobe and mode
    data = 0x2
    data = data | ( 0 << 14 )
    self.write_32b( BEEPER_MAIN_CR, data )
    
    # set run strobe and mode
    data = 0x2 
    data = data | ( 1 << 14 )
    self.write_32b( BEEPER_MAIN_CR, data )
    
 
  def freq_and_time( self, freq_hz, ms ):
    ms = ms
    self.set_freq( freq_hz )
    #self.set_time( ms )
    self.set_fpga_time( ms * 1000 )
  
  def pause( self, ms ):
    ms = ms
    time.sleep( ms / 1000.0 )
  
  def __init__( self ):
    global BEEPER_MAIN_CR, BEEPER_OSC_PHASE_W0_CR, BEEPER_OSC_PHASE_W1_CR, BEEPER_GEN_TIME_WO_CR, BEEPER_GEN_TIME_W1_CR
    self.dev = DeviceAccess()

    BEEPER_MAIN_CR         = self.CR( 0 )
    BEEPER_GEN_TIME_WO_CR  = self.CR( 1 )
    BEEPER_GEN_TIME_W1_CR  = self.CR( 2 )
    BEEPER_OSC_PHASE_W0_CR = self.CR( 3 )
    BEEPER_OSC_PHASE_W1_CR = self.CR( 4 )
    
    
  def beep( self, hz, time ):
    self.dev.hold_device( )
    self.freq_and_time( hz, time )
    self.pause( time )
    self.dev.release_device( )

  def imperial_march( self ):

    note_c   =  261                                                         
    note_d   =  294                                                         
    note_e   =  329                                                         
    note_f   =  349                                                         
    note_g   =  391                                                         
    note_gS  =  415                                                         
    note_a   =  440                                                         
    note_aS  =  455                                                         
    note_b   =  466                                                         
    note_cH  =  523                                                         
    note_cSH =  554                                                         
    note_dH  =  587                                                         
    note_dSH =  622                                                         
    note_eH  =  659                                                         
    note_fH  =  698                                                         
    note_fSH =  740                                                         
    note_gH  =  784                                                         
    note_gSH =  830                                                         
    note_aH  =  880

    Beeper().beep( note_a,  500 );
    Beeper().beep( note_a,  500 );
    Beeper().beep( note_a,  500 );
    Beeper().beep( note_f,  350 );
    Beeper().beep( note_cH, 150 );
    Beeper().beep( note_a,  500 );
    Beeper().beep( note_f,  350 );
    Beeper().beep( note_cH, 150 );
    Beeper().beep( note_a,  650 );

    time.sleep(0.150);
     
    Beeper().beep( note_eH, 500 );
    Beeper().beep( note_eH, 500 );
    Beeper().beep( note_eH, 500 );
    Beeper().beep( note_fH, 350 );
    Beeper().beep( note_cH, 150 );
    Beeper().beep( note_gS, 500 );
    Beeper().beep( note_f,  350 );
    Beeper().beep( note_cH, 150 );
    Beeper().beep( note_a,  650 );

    time.sleep(0.150);

    Beeper().beep( note_aH,  500 );
    Beeper().beep( note_a,   300 );
    Beeper().beep( note_a,   150 );
    Beeper().beep( note_aH,  400 );
    Beeper().beep( note_gSH, 200 );
    Beeper().beep( note_gH,  200 );
    Beeper().beep( note_fSH, 125 );
    Beeper().beep( note_fH,  125 );
    Beeper().beep( note_fSH, 250 );

    time.sleep(0.250);

    Beeper().beep( note_aS,  250 );
    Beeper().beep( note_dSH, 400 );
    Beeper().beep( note_dH,  200 );
    Beeper().beep( note_cSH, 200 );
    Beeper().beep( note_cH,  125 );
    Beeper().beep( note_b,   125 );
    Beeper().beep( note_cH,  250 );

    time.sleep(0.250);

    Beeper().beep( note_f,  125 );
    Beeper().beep( note_gS, 500 );
    Beeper().beep( note_f,  375 );
    Beeper().beep( note_a,  125 );
    Beeper().beep( note_cH, 500 );
    Beeper().beep( note_a,  375 );
    Beeper().beep( note_cH, 125 );
    Beeper().beep( note_eH, 650 );

    Beeper().beep( note_aH,   500 );
    Beeper().beep( note_a,    300 );
    Beeper().beep( note_a,    150 );
    Beeper().beep( note_aH,   400 );
    Beeper().beep( note_gSH,  200 );
    Beeper().beep( note_gH,   200 );
    Beeper().beep( note_fSH,  125 );
    Beeper().beep( note_fH,   125 );
    Beeper().beep( note_fSH,  250 );

    time.sleep(0.250);

    Beeper().beep( note_aS,  250 );
    Beeper().beep( note_dSH, 400 );
    Beeper().beep( note_dH,  200 );
    Beeper().beep( note_cSH, 200 );
    Beeper().beep( note_cH,  125 );
    Beeper().beep( note_b,   125 );
    Beeper().beep( note_cH,  250 );

    time.sleep(0.250);

    Beeper().beep( note_f,   250 );
    Beeper().beep( note_gS,  500 );
    Beeper().beep( note_f,   375 );
    Beeper().beep( note_cH,  125 );
    Beeper().beep( note_a,   500 );
    Beeper().beep( note_f,   375 );
    Beeper().beep( note_cH,  125 );
    Beeper().beep( note_a,   650 );


if __name__ == "__main__":
    beeper = Beeper()

    freq_list = [ 400, 800, 1000, 1200, 1500, 1800, \
                       2000, 2500, 3000, 3400 ]

    for freq in freq_list: 
      beeper.beep( freq, 500 )

    #Beeper().beep( 400, 100 )
    #Beeper().imperial_march( )
