import os

class Tca9535Lib( ):

  def __init__( self ):

    self.GPIO_PATH_PREFIX = "/sys/class/gpio"
    self.I2C_PREFIX = "/sys/class/i2c-dev/i2c-1/device/"

  ######################################################
  # Function inits GPIO in system                      #
  ######################################################
  def init_gpio( self, gpio_num ):

    export_path = "".join( [ self.GPIO_PATH_PREFIX, "/export" ] )
    self.write_to_file( export_path, str( gpio_num ) )

    gpio_dir = None

    for root, dirs, files in os.walk( self.GPIO_PATH_PREFIX ):
      if dirs and gpio_dir is None:
        for d in dirs:
          if d.startswith( "gpio" + str( gpio_num ) ):
            gpio_dir = d
            
    if gpio_dir is None:
      print "Error: GPIO %d initialization fail" % ( gpio_num )
      return 1

    self.gpio_conf_pin_input( gpio_num )

    return 0

  ######################################################
  # Function deinits GPIO from system                  #
  ######################################################
  def deinit_gpio( gpio_num ):
    unexport_path = "".join( [ self.GPIO_PATH_PREFIX, "/unexport" ] )
    self.write_to_file( unexport_path, str( gpio_num ) )

  ######################################################
  # Function finds GPIO expander by that symbolic name #
  ######################################################
  def find_expander( self, expander_addr ):
    path = None

    path  = "".join( [ self.I2C_PREFIX, "/1-00" + str( expander_addr ) + "/" ] )
    
    chip_dir = None

    for root, dirs, files in os.walk( path ):
      if dirs and chip_dir is None:
        for d in dirs:
          if d.startswith( "gpiochip" ):
            chip_dir = d
            
    if chip_dir is None:
      print "Can't find path to %s expander" %( expander_addr )
      return None
    
    gpio_path = "".join( [ self.GPIO_PATH_PREFIX + "/" + chip_dir ] ) 
    
    return gpio_path

  ######################################################
  # Function gets GPIO pin count for an expander       #
  ######################################################
  def get_gpios_cnt( self, gpiochip_addr ):
    path = "".join( [ self.GPIO_PATH_PREFIX, "/gpiochip", str( gpiochip_addr ), "/npins" ] )

    return self.read_from_file( path )

  ######################################################
  # Function calculateses a GPIO pin system number     #
  # using chip number and GPIO relative index          #
  ######################################################
  def calc_gpio_number( self, gpiochip_addr, gpio_rel_num ):
    path = "".join( [ gpiochip_addr, "/base" ] )
    base = self.read_from_file( path ) 
    
    return ( int( base ) + gpio_rel_num )
  
  def write_to_file( self, fname, data ):
    f = open( fname, 'w' )
    f.write( data )
    f.close( )

  def read_from_file( self, fname ):
    f = open( fname, 'r' )
    rd_data = f.read( )
    f.close( )

    rd_data = rd_data.rsplit( )

    # we reading stuff with only one word, 
    # so just interested in one word
    return rd_data[0]

  ######################################################
  # Function configures gpio as input                  #
  ######################################################
  def gpio_conf_pin_input( self, gpio_num ):
    path = "".join( [ self.GPIO_PATH_PREFIX, "/gpio", str( gpio_num ), "/direction" ] )
    
    self.write_to_file( path, "in" )

  ######################################################
  # Function configures gpio as output                 #
  ######################################################
  def gpio_conf_pin_output( self, gpio_num ):
    path = "".join( [ self.GPIO_PATH_PREFIX, "/gpio", str( gpio_num ), "/direction" ] )
    
    self.write_to_file( path, "out" )

  ######################################################
  # Function gets gpio direction                       #
  ######################################################
  def gpio_get_pin_direction( self, gpio_num ):
    path = "".join( [ self.GPIO_PATH_PREFIX, "/gpio", str( gpio_num ), "/direction" ] )

    return self.read_from_file( path )

  ######################################################
  # Function sets gpio to logical 0 or 1               #
  ######################################################
  def gpio_set_pin( self, gpio_num, level ):
    path = "".join( [ self.GPIO_PATH_PREFIX, "/gpio", str( gpio_num ), "/value" ] )

    self.write_to_file( path, str( level ) ) 

  ######################################################
  # Function sets gpio to logical 1                    #
  ######################################################
  def gpio_set_pin_high( self, gpio_num ):
    self.gpio_set_pin( gpio_num, 1 )

  ######################################################
  # Function sets gpio to logical 0                    #
  ######################################################
  def gpio_set_pin_low( self, gpio_num ):
    self.gpio_set_pin( gpio_num, 0 )

  ######################################################
  # Function gets gpio value                           #
  ######################################################
  def gpio_get_pin( self, gpio_num ):
    path = "".join( [ self.GPIO_PATH_PREFIX, "/gpio", str( gpio_num ), "/value" ] )
    data = self.read_from_file( path )

    return data
