#!/bin/sh -efu

FIRMWARECONFDIR="${FIRMWARECONFDIR:-/etc/rootfs-utils}"
FIRMWARECONF="${FIRMWARECONF:-$FIRMWARECONFDIR/firmware.conf}"

if [ -n "$FIRMWARECONF" -a -e "$FIRMWARECONF" ]; then
    . "$FIRMWARECONF"
fi





# ----------------------------------------------- #
# Write U-Boot environment to device.             #
#                                                 #
# U-Boot environment is written right after MBR   #
# (skipping first 512 bytes)                      #
# ----------------------------------------------- #
ubootenv_command()
{
    echo "Writing U-Boot environment"

    if [ $# -ne 2 ]; then
        echo "Usage: craft ubootenv <device> <uboot-env image>"
        exit 1
    fi

    DEVICE=$1
    ENVFILE=$2

    if [ ! -b $DEVICE ]; then
        echo "$DEVICE is not a block device"
        exit 1
    fi

    if [ ! -f $ENVFILE ]; then
        echo "Can't open U-Boot environment file '$ENVFILE'"
        exit 1
    fi

    dd of=${DEVICE} if=${ENVFILE} bs=1 seek=512
    sync
    echo "U-Boot environment '$ENVFILE' was written to '$DEVICE'"
}


# ---------------------------------------------------------------------- #
# Write U-Boot image to device                                           #
#                                                                        #
# U-Boot image is written into first partition after preloader,          #
# so it's generally good idea to first write preloader and then U-Boot.  #
# ---------------------------------------------------------------------- #
uboot_command()
{
    echo "Writing U-boot image"

    if [ $# -ne 2 ]; then
        echo "Usage: craft uboot <device> <uboot image file>"
        exit 1
    fi

    DEVICE=${1}p1 # First partition
    IMAGE=$2

    if [ ! -b $DEVICE ]; then
        echo "$DEVICE is not a block device"
        exit 1
    fi

    if [ ! -f $IMAGE ]; then
        echo "Can't open U-Boot image '$IMAGE'"
        exit 1
    fi

    dd of=${DEVICE} if=${IMAGE} bs=64k seek=4
    sync
    echo "U-Boot image '$IMAGE' was written to '$DEVICE'"
}

# ---------------------------------------------------- #
# Write Preloader image to device                      #
#                                                      #
# Preloader is written into first partition of device  #
# ---------------------------------------------------- #
preloader_command()
{
    echo "Writing preloader image"

    if [ $# -ne 2 ]; then
        echo "Usage: craft preloader <device> <preloader image>"
        exit 1
    fi

    DEVICE=${1}p1 # First partition
    IMAGE=$2

    if [ ! -b $DEVICE ]; then
        echo "$DEVICE is not a block device"
        exit 1
    fi

    if [ ! -f $IMAGE ]; then
        echo "Can't open preloader image '$IMAGE'"
        exit 1
    fi

    dd oflag=direct of=${DEVICE} if=${IMAGE}
    echo "Preloader image '$IMAGE' was written to '$DEVICE'"
}











