#!/usr/bin/python

import sys, getopt, json

from txtstruct import *
from mtktests  import *
from mtkstrings import *

######################## READING THE OPTIONS ########################

check_locale_completeness(strings_en, strings_ru)

def main():

  def print_help():
    print 'Usage: ' + sys.argv[0]
    print '  -h      --help          Print this help message'
    print '  -T TYPE --test=TYPE     Generate report for test TYPE'
    print '  -t TYPE --type=TYPE     Generate report in format TYPE'
    print '                          (can occur several times)'
    print '  -l LANG --language=LANG Use locale LANG in text'
    print '  -d DIR  --directory=DIR Output to directory DIR'
    print '  -p PRF  --prefix=PRF    Use prefix PRF for file name'
    print '  -f FILE --file=FILE     Read file instead of stdin'

  argv = sys.argv[1:]
  try:
    opts, args = getopt.getopt(argv, 'hT:t:d:p:l:f:',
      ['test=', 'type=', 'directory=', 'prefix=', 'language=',
       'file=', 'help'])
  except getopt.GetoptError:
    print_help()
    sys.exit(2)

  prefix = 'report'
  directory = None
  types = []
  lang = 'en'
  test = None
  infile = None

  for opt, arg in opts:
    if opt in ('--help', '-h'):
      print_help()
      sys.exit()
    elif opt in ('--test', '-T'):
      test = arg
    elif opt in ('--type', '-t'):
      types.append(arg)
    elif opt in ('--directory', '-d'):
      directory = arg
    elif opt in ('--prefix', '-p'):
      prefix = arg
    elif opt in ('--language', '-l'):
      lang = arg
    elif opt in ('--file', '-f'):
      infile = arg

  if len(types) == 0:
    types = ['pdf', 'docx', 'html']

  strings = None
  if lang == 'en':
    strings = strings_en
  elif lang == 'ru':
    strings = strings_ru
  else:
    raise NotImplementedError('Unsupported language ' + lang)

  if infile is None:
    j = json.loads(sys.stdin.read())
  else:
    with open(infile, 'r') as f:
      j = json.loads(f.read())

  rs = None

  if directory is None:
    path = prefix
  else:
    path = directory + '/' + prefix

  if test == 'bert':
    cl = BerTest
  elif test == 'rfc2544':
    cl = Rfc2544
  elif test == 'y1564':
    cl = Y1564Test
  elif test == 'vct':
    cl = VcTest
  elif test is None:
    cl = MtkReport
  else:
    raise NotImplementedError('Unsupported test ' + test)

  rs = Document(cl.from_dict(j).to_report(strings))

  for tp in types:
    if tp == 'pdf':
      rs.format_full(FormatTypes.PDF,  path + '.pdf')
    elif tp == 'docx':
      rs.format_full(FormatTypes.DOCX, path + '.docx')
    elif tp == 'html':
      rs.format_full(FormatTypes.HTML, path + '.html')

main()

